
#import <Aluminum/RendererOSX.h>
#import <Aluminum/MeshBuffer.hpp>
#import <Aluminum/MeshUtils.hpp>
#import <Aluminum/Program.hpp>
#import <Aluminum/Texture.hpp>
#import <Aluminum/ResourceHandler.h>
#include "Aluminum/FBO.hpp"

#define T 4

/*
 Demonstrates the use of simple convolution filter in the fragment shader for a) edge detection and b) embossing an image. Lots of somewhat unintuitive controls via keyboard and mouse, so look at the keyDown and moveMoved methods.
 */

using namespace aluminum;

class SketchFilter : public RendererOSX {
    
public:
    
    mat4 model, view, proj;
    
    
    Program program, gaussProgram, gaussVertical, normal;
    
    GLint posLoc=0;
    GLint texCoordLoc=1;
    FBO fbo, fbo2;
    
    Texture textures[T];
    MeshBuffer mb1, mb2;
    ResourceHandler rh;
    
    float weight = .5;
    float k = .5;
    float randomNumber;
    float beta = .5;
    float epsilon = .01;
    float alfa = .1;
    float gamma = 0.8;
    int which = 0;
    int mappingActive = 0;
    int edgeActive = 0;
    int gaussActive = 0;
    
    void onCreate() {
        //For the black and white filter use this shader:
        rh.loadProgram(program, "texture", posLoc, -1, texCoordLoc, -1);
        
        //For the colored version use this shader:
        //rh.loadProgram(program, "colored", posLoc, -1, texCoordLoc, -1);
        rh.loadProgram(gaussProgram,"gauss",posLoc,-1,texCoordLoc,-1);
        rh.loadProgram(gaussVertical, "gaussVertical", posLoc, -1, texCoordLoc, -1);
        rh.loadProgram(normal, "normal", posLoc, -1, texCoordLoc, -1);
        

        rh.loadTexture(textures[0], "isoleborromee.jpg");
        rh.loadTexture(textures[1], "Chicago.jpg");
        rh.loadTexture(textures[2], "val-funes.jpg");
        rh.loadTexture(textures[3], "montagna.jpg");
        
        
        fbo.create(800, 600);
        fbo2.create(800, 600);
        
        mb1.init(MeshUtils::makeRectangle(2.0, 2.0), posLoc, -1, texCoordLoc, -1);
        mb2.init(MeshUtils::makeRectangle(2.0, 2.0), posLoc, -1, texCoordLoc, -1);
        proj = glm::perspective(glm::radians(60.0), 1.0, 0.1, 100.0);
        view = glm::lookAt(vec3(0,0,2), vec3(0,0,0), vec3(0,1,0) );
        model = glm::mat4(1.0);
        
        glEnable(GL_DEPTH_TEST);
        glViewport(0, 0, width, height);
        glClearColor(0.3,0.3,0.3,1.0);
    }
    
    void onFrame(){
        
        
        textures[which].wrapMode(GL_REPEAT);
        
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        fbo.bind();{
            glViewport(0, 0, fbo.width, fbo.height);
            glClearColor(0.1,0.1,0.1,1.0);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
            program.bind(); {
                
                glUniformMatrix4fv(program.uniform("model"), 1, 0, ptr(model));
                glUniformMatrix4fv(program.uniform("view"), 1, 0, ptr(view));
                glUniformMatrix4fv(program.uniform("proj"), 1, 0, ptr(proj));
                
                float dx = 1.0/textures[which].width;
                float dy = 1.0/textures[which].height;
                

                glUniform1f(program.uniform("gamma"), gamma);
                glUniform1f(program.uniform("alfa"), alfa);
                glUniform1i(program.uniform("mappingActive"), mappingActive);
                glUniform1f(program.uniform("epsilon"), epsilon);
                glUniform1f(program.uniform("beta"), beta);
                glUniform1i(program.uniform("edgeActive"), edgeActive);
                glUniform1f(program.uniform("k"),k);
                glUniform1f(program.uniform("weight"),weight);
                glUniform1f(program.uniform("rendomNumber"),randomNumber);
                
                
                glUniform1f(program.uniform("dx"), dx );
                glUniform1f(program.uniform("dy"), dy );
                glUniform1i(program.uniform("tex0"), 0);
                
                textures[which].bind(GL_TEXTURE0); {
                    mb1.draw();
                } textures[which].unbind(GL_TEXTURE0);
                
                
                cout << "width: " << textures[which].width <<endl;
                cout << "height: " << textures[which].height <<endl;
                cout << "alfa: " << alfa <<endl;
                cout << "mapping active" << mappingActive <<endl;
                cout << "edge active: " << edgeActive <<endl;
                cout << "k value: " << k <<endl;
                cout << "weight: " << weight <<endl;
                cout << "epsilon: " << epsilon <<endl;
            } program.unbind();
        }fbo.unbind();
        
        
//        if(gaussActive == 1){
            fbo2.bind();{
              glViewport(0, 0, fbo.width, fbo.height);
              glClearColor(0.0,0.0,0.0,1.0);
              glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
                
                gaussProgram.bind();{
                    glUniformMatrix4fv(gaussProgram.uniform("model"), 1, 0, ptr(model));
                    glUniformMatrix4fv(gaussProgram.uniform("view"), 1, 0, ptr(view));
                    glUniformMatrix4fv(gaussProgram.uniform("proj"), 1, 0, ptr(proj));
                    
                    
                    glUniform1i(gaussProgram.uniform("tex0"), 0);
                    glUniform1i(gaussProgram.uniform("gaussActive"), gaussActive);
                    
                    //fbo.texture.wrapMode(GL_REPEAT);
                    fbo.texture.bind(GL_TEXTURE0); {
                        mb1.draw();
                    } fbo.texture.unbind(GL_TEXTURE0);
                    
                }gaussProgram.unbind();
            }fbo2.unbind();
            
            
//            glViewport(0, 0, width, height);
//            glClearColor(0.0,0.0,0.0,1.0);
//            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
            
            gaussVertical.bind();{
                glUniformMatrix4fv(gaussVertical.uniform("model"), 1, 0, ptr(model));
                glUniformMatrix4fv(gaussVertical.uniform("view"), 1, 0, ptr(view));
                glUniformMatrix4fv(gaussVertical.uniform("proj"), 1, 0, ptr(proj));
                
                glUniform1i(gaussVertical.uniform("tex0"), 0);
                glUniform1i(gaussVertical.uniform("gaussActive"), gaussActive);
                
                //fbo2.texture.wrapMode(GL_REPEAT);
                fbo2.texture.bind(GL_TEXTURE0); {
                    mb1.draw();
                } fbo2.texture.unbind(GL_TEXTURE0);
                
            }gaussVertical.unbind();

        
        
    }
    
//    void onReshape() {
//        glViewport(0, 0, width, height);
//    }
    
    
    virtual void keyDown(char key) {
        
        switch(key) {
            case kVK_ANSI_S :
                alfa += .01;
                if(alfa > 1 ){
                    alfa = 1;
                }
                break;
                
            case kVK_ANSI_X :
                alfa -= .01;
                if(alfa < 0 ){
                    alfa = 0;
                }
                break;
                
            case kVK_ANSI_C :
                ++mappingActive %= 3;
                break;
                
            case kVK_ANSI_R :
                epsilon += .01;
                if(epsilon > 1 ){
                    epsilon = 1;
                }
                break;
                
            case kVK_ANSI_F :
                epsilon -= .01;
                if(epsilon < 0 ){
                    epsilon = 0;
                }
                break;
                
            case kVK_ANSI_T :
                beta += .001;
                if(beta > 1 ){
                    beta = 1;
                }
                break;
                
            case kVK_ANSI_G :
                beta -= .001;
                if(beta < 0 ){
                    beta = 0;
                }
                break;
                
            case kVK_ANSI_Y:
                k += .01;
                if(k > 1 ){
                    k = 1;
                }
                break;
            case kVK_ANSI_H:
                k -= .01;
                if(k < 0 ){
                    k = 0;
                }
                break;
                
            case kVK_ANSI_U:
                weight += .01;
                if(weight > 1 ){
                    weight = 1;
                }
                break;
            case kVK_ANSI_J:
                weight -= .01;
                if(weight < 0 ){
                    weight = 0;
                }
                break;
                
                
            case kVK_ANSI_L:
                if(gaussActive == 0)
                    gaussActive = 1;
                else
                    gaussActive = 0;
                break;
            case kVK_Space :
                ++which %= T;
                break;
                
                
        }
        
    }
    
};


int main() {
    return SketchFilter().start("aluminum::SketchFilter", 100, 100, 800, 600);
}
